@@ -17,19 +17,26 @@ class PredicateLanguage(Language):
     ----------
     individual_constants: list of str, optional
         A list of strings (valid letters for individual constants). Can also be a callable (a function).
+    individual_metavariables: list of str, optional
+        A list of strings. Can be instantiated with ind cts. Are part of the metalanguage, any
+        formula containing them is schematic. Useful for formulating rules.
     variables: list of str, optional
-        A list of strings (valid letters for variables). Will consider any variable + a digit a valid variable.
+        A list of strings (valid letters for variables). Will consider any variable + a digit a valid variable. Are part
+        of the base language, can appear quantified over in non-schematic formulae.
+    variable_metavariables: list of str, optional
+        Same as individual_metavariables, but can be instantiated with variables.
     quantifiers: list of str, optional
         A list of strings (valid letters for quantifiers)
     metavariables: list of str, optional
-        A list of strings (valid letters for atomic metavariables)
+        A list of strings (valid letters for sentential metavariables)
     constant_arity_dict: dict, optional
         a dictionary that contains `str` (logical cosntant symbols) as keys, and positive `int` (arities) as values
     predicate_letters: dict, optional
         a dictionary that contains `str` (predicate letter symbols) as keys, and positive `int` (arities) as values
     predicate_variables: dict, optional
         a dictionary that contains `str` (predicate variable symbols) as keys, and positive `int` (arities) as values.
-        Will consider any variable + a digit a valid variable.
+        Will consider any variable + a digit a valid variable. Are part of the base language, , can appear quantified
+        over in non-schematic formulae.
     function_symbols: dict, optional
         a dictionary that contains `str` (function symbols) as keys, and positive `int` (arities) as values
     sentential_constants: list of str, optional
@@ -39,7 +46,7 @@ class PredicateLanguage(Language):
 
     Examples
     --------
-    Example language containing all of the above
+    Example language containing some of the above
 
     >>> from logics.classes.predicate import PredicateLanguage
     >>> language = PredicateLanguage(individual_constants=['a', 'b', 'c', 'd'],
@@ -112,12 +119,15 @@ class PredicateLanguage(Language):
     """
 
 
-    def __init__(self, individual_constants=None, variables=None, quantifiers=None, metavariables=None,
-                 constant_arity_dict=None, predicate_letters=None, predicate_variables=None, function_symbols=None,
-                 sentential_constants=None, allow_predicates_as_terms = False):
+    def __init__(self, individual_constants=None, individual_metavariables=None, variables=None,
+                 variable_metavariables=None, quantifiers=None, metavariables=None, constant_arity_dict=None,
+                 predicate_letters=None, predicate_variables=None, function_symbols=None, sentential_constants=None,
+                 allow_predicates_as_terms=False):
         self.atomics = None  # Used in (propositional) Language but not here
         self.individual_constants = individual_constants or []
         self.variables = variables or []
+        self.variable_metavariables = variable_metavariables or []
+        self.individual_metavariables = individual_metavariables or []
         self.quantifiers = quantifiers
         self.metavariables = metavariables or []
         self.constant_arity_dict = constant_arity_dict or {}
@@ -144,7 +154,8 @@ class PredicateLanguage(Language):
             return {c for c in self.predicate_letters if self.predicate_letters[c] == arity}
 
     def arity(self, string):
-        """Overriden from the base class to include the arities of predicate letters/variables and sentential constants
+        """Overriden from the base class to include the arities of predicate letters/variables/metavariables and
+        function symbols
 
         Raises
         ------
@@ -164,6 +175,8 @@ class PredicateLanguage(Language):
         2
         >>> classical_function_language.arity('f')  # function symbol
         1
+        >>> classical_function_language.arity('Π')  # predicate metavariable
+        1
         """
         if string in self.constant_arity_dict:
             return self.constant_arity_dict[string]
@@ -171,6 +184,7 @@ class PredicateLanguage(Language):
             return self.predicate_letters[string]
         elif string in self.function_symbols:
             return self.function_symbols[string]
+        # Predicate variables can contain a digit afterwards
         for v in self.predicate_variables:
             if string == v:
                 return self.predicate_variables[v]
@@ -186,7 +200,7 @@ class PredicateLanguage(Language):
                     (term in self.predicate_letters or term in self.predicate_variables or
                      term in self.function_symbols):
                 return True
-            return self._is_valid_individual_constant_or_variable(term)
+            return self._is_valid_individual_constant_or_variable(term)  # includes individual metavariables
 
         # If not a string, it must be a tuple, of the form ('f', term, term, ...)
         else:
@@ -205,8 +219,8 @@ class PredicateLanguage(Language):
             return True
 
     def _is_atomic_well_formed(self, formula, return_error):
-        # Metavariables or sentential constants
-        if self.is_metavariable_string(formula[0]) or self.is_sentential_constant_string(formula[0]):
+        # Sentential metavariables and constants
+        if formula[0] in self.metavariables or self.is_sentential_constant_string(formula[0]):
             if not return_error:
                 return True
             return True, ''
@@ -236,7 +250,7 @@ class PredicateLanguage(Language):
     def _is_molecular_well_formed(self, formula, return_error):
         # We only need to take into account the case of quantified formulae, the rest is the same
         if formula.main_symbol in self.quantifiers:
-            if not self._is_valid_variable(formula[1]):
+            if not self._is_valid_variable(formula[1], allow_metavariables=True):  # includes ind and pred metavariables
                 if not return_error:
                     return False
                 return False, f'{formula} is not well-formed: {formula[1]} is not a valid variable'
@@ -254,36 +268,57 @@ class PredicateLanguage(Language):
         # Non-quantified cases, call the super method
         return super()._is_molecular_well_formed(formula, return_error)
 
+    def is_atomic_string(self, string):
+        raise ValueError('Method only available in propositional languages')
+
     def _is_valid_predicate(self, string):
-        return self._is_valid_variable(string, only_predicate=True) or string in self.predicate_letters
+        return self._is_valid_variable(string, only_predicate=True, allow_metavariables=True) or \
+            string in self.predicate_letters
 
     def _is_valid_function_symbol(self, string):
         return string in self.function_symbols
 
     def is_valid_individual_constant(self, string):
+        if string in self.individual_metavariables:
+            return True
         if callable(self.individual_constants):
             return self.individual_constants(string)
         return string in self.individual_constants
 
-    def _is_valid_variable(self, string, only_individual=False, only_predicate=False):
+    def _is_valid_variable(self, string, only_individual=False, only_predicate=False, allow_metavariables=True):
         """There is always an infinite supply of both individual and predicate variables"""
         if only_individual and only_predicate:
             raise ValueError("only_individual and only_predicate parameters cannot be both True")
         elif only_individual:
             bound = self.variables
+            meta_bound = self.variable_metavariables
         elif only_predicate:
             bound = self.predicate_variables
+            meta_bound = []
         else:
             bound = chain(self.variables, self.predicate_variables)
+            meta_bound = self.variable_metavariables
         for vv in bound:
             if string == vv:
                 return True
             if string[:len(vv)] == vv and string[len(vv):].isdigit():
                 return True
+        if allow_metavariables:
+            for mv in meta_bound:
+                if string == mv:
+                    return True
+                # these canot have digits after
         return False
 
     def _is_valid_individual_constant_or_variable(self, string):
-        return self.is_valid_individual_constant(string) or self._is_valid_variable(string, only_individual=True)
+        return self.is_valid_individual_constant(string) or self._is_valid_variable(string, only_individual=True,
+                                                                                    allow_metavariables=True)
+
+    def is_metavariable_string(self, string):
+        """Determines if a string is among the (individual/predicate/formula) metavariables of the language."""
+        return string in self.metavariables or \
+            string in self.individual_metavariables or \
+            string in self.variable_metavariables
 
 
 class InfinitePredicateLanguage(PredicateLanguage, InfiniteLanguage):
